package ru.amse.kanzheleva.moviemaker.player;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Timer;

import ru.amse.kanzheleva.moviemaker.movie.IMovie;
import ru.amse.kanzheleva.moviemaker.ui.frame.JDrawer;

public class MoviePlayer extends JDrawer {
	private static final long serialVersionUID = 1L;
	private IMovie myMovie;
	private Timer myTimer;
	private boolean myIsFrozen;
	private int myFrameNumber;
	private final MovieBox myOwner;
		
	
	public MoviePlayer(MovieBox owner, IMovie movie, int width, int height, double scale,
			double scale2, int fps) {
		super(movie.getFrame(0), width, height, scale, scale2);
		myOwner = owner;
		myMovie = movie;
		myFrameNumber = -1;
		myIsFrozen = true;
		myTimer = new Timer(1000 / fps, new TimerListener());
	}
	
	public void setNewFrame(int frameNumber) {
		setFrame(myMovie.getFrame(frameNumber));
		myFrameNumber = frameNumber;
	}
	
	
	public void setNewMovie(IMovie movie) {
		myMovie = movie;
		myFrameNumber = -1;
		myIsFrozen = true;
		myTimer.setDelay(1000 / movie.getFPS());
		myOwner.ableMove(true);

	}
	
	public void stopAnimation() {
        if (myTimer.isRunning()) {
        	myTimer.stop();
        }
	}
	
	public void startAnimation() {
		if (myIsFrozen) {
	    } else {
	       if (!myTimer.isRunning()) {
	           myTimer.start();
	       }
	    }
	}
	
	public void setNewDelay(int fps) {
		if (fps != 0) {
			myTimer.setDelay(1000 / fps);
			myOwner.ableMove(true);
		} else {
			myOwner.ableMove(false);
		}
	}
	
	public boolean isFrozen() {
		return myIsFrozen;
	}
	
	public void setIsFrozen(boolean isFrozen) {
		myIsFrozen = isFrozen;
	}
	
	private class TimerListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			myOwner.moveTimeSlider();
			if (myFrameNumber < myMovie.getSize()) {
				setFrame(myMovie.getFrame(myFrameNumber));
				myFrameNumber++;
			} else {
				myFrameNumber = 0;
				setFrame(myMovie.getFrame(myFrameNumber));
				myIsFrozen = true;
				myOwner.resetTimeSliderAndButton();
				stopAnimation();
			}
		}		
	}
	
	
	
}