package ru.amse.kanzheleva.moviemaker.player;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class SpeedDialog extends JDialog {
	private static final long serialVersionUID = 1L;
	private final JSlider myFramesPerSecondSlider;
	private int myNewFps;
	private int myOldFps;

	public SpeedDialog(JFrame parent, String title, int fps) {
		super(parent, title, false);
		setModal(true);
		setSize(new Dimension(300, 120));
 
		JButton okButton = makeOkButton();
		JButton cancButton = makeCancelButton();
		myFramesPerSecondSlider = makeFramePerSecondSlider(fps);
		myNewFps = fps;
		myOldFps = fps;
		
		JPanel buttonsPanel = new JPanel();
		buttonsPanel.setLayout(new BoxLayout(buttonsPanel, BoxLayout.X_AXIS));
		buttonsPanel.add(okButton);
		buttonsPanel.add(cancButton);
			 
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		contentPane.add(myFramesPerSecondSlider);
		contentPane.add(buttonsPanel);
		add(contentPane);
	}
	
	public void setFPS(int fps) {
		myNewFps = fps;
		myOldFps = fps;
		myFramesPerSecondSlider.setValue(fps);
	}

	private JSlider makeFramePerSecondSlider(int fps) {
		 JSlider framesPerSecondSlider = new JSlider(JSlider.HORIZONTAL, 0, 40, fps);
		 framesPerSecondSlider.addChangeListener(new FPSSliderListener());
         framesPerSecondSlider.setMajorTickSpacing(10);
         framesPerSecondSlider.setMinorTickSpacing(1);
         framesPerSecondSlider.setPaintTicks(true);
         framesPerSecondSlider.setPaintLabels(true);
         framesPerSecondSlider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
         return framesPerSecondSlider;
	}
	
	private JButton makeOkButton() {
		JButton okButton = new JButton("OK");
		 okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				myFramesPerSecondSlider.setValue(myNewFps);
				myOldFps = myNewFps;
				SpeedDialog.this.setVisible(false);
			}		 
		 });
		 okButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		return okButton;
	}
	
	public int openDialog() {
		setVisible(true);
		return myFramesPerSecondSlider.getValue();
	}
	
	private JButton makeCancelButton() {
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				myFramesPerSecondSlider.setValue(myOldFps);
				SpeedDialog.this.setVisible(false);
			}
		});
		return cancelButton;
	}
	
	private class FPSSliderListener implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            JSlider source = (JSlider)e.getSource();
            if (!source.getValueIsAdjusting()) {
                myNewFps = (int)source.getValue();
            }
        }
    }   
	
}
