package ru.amse.kanzheleva.moviemaker.ui.frame;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.Shape;
import java.util.List;

import javax.swing.JComponent;

import ru.amse.kanzheleva.moviemaker.movie.IFrame;
import ru.amse.kanzheleva.moviemaker.movie.IGraphicalObject;
import ru.amse.kanzheleva.moviemaker.ui.visitor.ShapeMaker;

public class JDrawer extends JComponent {
	private static final long serialVersionUID = 1L;
	private List<IGraphicalObject> myListFrame;
	protected int myWidth;
	protected int myHeight;
	private double myXScale;
	private double myYScale;
	
	public JDrawer(IFrame frame, int width, int height, double xScale, double yScale) {
		myWidth = width;
		myHeight = height;
		myXScale = xScale;
		myYScale = yScale;
		myListFrame = frame.getGraphicalObjects();
		setBackground(Color.white);
		
	}
	
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		
		AffineTransform at = AffineTransform.getScaleInstance(myXScale, myYScale);
		g2.transform(at);
		
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setColor(Color.white);
		g2.fill(new Rectangle2D.Double(0, 0, myWidth, myHeight));
		 
		for (IGraphicalObject element : myListFrame) {
			drawElement(g2, element);
		}
	}
	
	public void setFrame(IFrame frame) {
		myListFrame = frame.getGraphicalObjects();
		repaint();
	}
	
	private void drawElement(Graphics2D g2, IGraphicalObject element) {
		Shape s = element.visit(ShapeMaker.getInstance(), null);
		
		AffineTransform last = g2.getTransform();
		
		AffineTransform at = AffineTransform.getRotateInstance(element.getRotationAngle(),
				element.getRotationCenter().getX(),
				element.getRotationCenter().getY());
		g2.transform(at);
		
		g2.setColor(new Color(element.getFillColor().getRed(), 
				element.getFillColor().getGreen(), element.getFillColor().getBlue())/* new Color(0, 255, 0)*/);
		g2.fill(s);
		
		g2.setColor(new Color (element.getLineColor().getRed(), 
				element.getLineColor().getGreen(), element.getLineColor().getBlue())/*new Color(0, 255, 255)*/);
		g2.draw(s);
		
		g2.setTransform(last);		
	}
	
}
