package ru.amse.kanzheleva.moviemaker.ui.frameeditor;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Shape;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.Point2D;
import java.util.LinkedList;
import java.util.List;

import ru.amse.kanzheleva.moviemaker.movie.ColorRGB;
import ru.amse.kanzheleva.moviemaker.movie.IFrame;
import ru.amse.kanzheleva.moviemaker.movie.IGraphicalObject;
import ru.amse.kanzheleva.moviemaker.movie.Moviable;
import ru.amse.kanzheleva.moviemaker.movie.figures.PolyFigure;
import ru.amse.kanzheleva.moviemaker.movie.figures.RectangularFigure;
import ru.amse.kanzheleva.moviemaker.ui.frame.JDrawer;
import ru.amse.kanzheleva.moviemaker.ui.frame.MovePointsListMaker;
import ru.amse.kanzheleva.moviemaker.ui.visitor.ShapeMaker;

public class FrameEditor extends JDrawer {
	private static final long serialVersionUID = 1L;
	private IFrame myFrame;
	private List<Shape> myShapeList;
	private int myUsedFigure = -1;
	private List<IGraphicalObject> myFiguresList;
	private int myOldX = 0;
	private int myOldY = 0;
	private IState myState;
	private List<Moviable> myMovePoints;
	private List<Shape> myMovePointsShapeList;
	private int myPoint = -1;
	private IChangaListener myChangaListener;
	
	private BeforeFigureChosen beforeFigureChosen = new BeforeFigureChosen();
	private FigureChosen figureChosen = new FigureChosen();
	private FigureMove figureMove = new FigureMove();
	private FigureResize figureResize = new FigureResize();
	
	private int myWidth;
	private int myHeight;
	
	public FrameEditor(IFrame frame, int width, int height, double xScale, double yScale) {
		super(frame, width, height, xScale, yScale);
		myFrame = frame;
		myWidth = width;
		myHeight = height;
		addMouseMotionListener(new MouseDraggedListener());
		addMouseListener(new MouseActionListener());
		fillShapeList();
		myUsedFigure = -1;
		myFiguresList = myFrame.getGraphicalObjects();
		myState = beforeFigureChosen;
	
	}
	
	public void addChangeListener(IChangaListener changeListaner) {
		myChangaListener = changeListaner;
	}
	
	private void chooseFigure(int number) {
		myUsedFigure = number;
		if (number == -1) {
			myChangaListener.figureSelected(false);
		} else {
			myChangaListener.figureSelected(true);
		}
	}
	
	public void setFrame(IFrame frame) {
		myFrame = frame;
		super.setFrame(frame);
		fillShapeList();
		chooseFigure(-1);
	}

	public void addFigure(IGraphicalObject figure, int n) {
		chooseFigure(-1);
		figure.visit(new FigureFieldsFiller(), n);
		myFrame.addFigure(figure);
		fillShapeList();
		repaint();
		myChangaListener.frameChange();
	}
	
	public void setFillColor(Color color) {
		if (myUsedFigure != -1) {
			myFiguresList.get(myUsedFigure).setFillColor(new ColorRGB(color.getRed(),
					 color.getGreen(), color.getBlue()));
			repaint();
		}
	}
	
	public void setLineColor(Color color) {
		if (myUsedFigure != -1) {
			myFiguresList.get(myUsedFigure).setLineColor(new ColorRGB(color.getRed(),
					 color.getGreen(), color.getBlue()));
			repaint();
		}
	}
	
	private void fillShapeList() {
		myShapeList = new LinkedList<Shape>();
		myFiguresList = myFrame.getGraphicalObjects();
		for (IGraphicalObject element : myFiguresList) {
			myShapeList.add(element.visit(ShapeMaker.getInstance(), null));			
		}
	}
	
	private void fillMovePointsList() {
		myMovePointsShapeList = new LinkedList<Shape>();
		for (Moviable point: myMovePoints) {
			myMovePointsShapeList.add(point.visit(ShapeMaker.getInstance(), null));
		}
	}
	
	
	private int searchChosenMovePoint(Point2D point) {
		int pointNumber = -1;
		for (Shape shape: myMovePointsShapeList) {
			pointNumber++;
			if (shape.contains(point)) {
				return pointNumber;
			} 
		}
		return -1;
	}
	
	private int searchChosenFigure(Point2D point) {
		int frameNumber = -1;
		int newUsedFigure = -1;
		for (Shape shape: myShapeList) {
			frameNumber++;
			if (shape.contains(point)) {
				newUsedFigure = frameNumber;
			}
		}
		return newUsedFigure;
	}
	
	public void changeShapeInList(int index) {
		myShapeList.remove(index);
		myShapeList.add(myFiguresList.get(index).visit(ShapeMaker.getInstance(), null));
	}
	
	private void setOldValueAndFillList(Point point) {
		myOldX = (int) point.getX();
		myOldY = (int) point.getY();
		changeShapeInList(myUsedFigure);
		IGraphicalObject figure = myFiguresList.get(myUsedFigure);
		myFiguresList.remove(myUsedFigure);
		myFiguresList.add(myFiguresList.size(), figure);
		chooseFigure(myFiguresList.size() - 1);
		myMovePoints = figure.visit(MovePointsListMaker.getInstance(), null);
		fillMovePointsList();
		repaint();
	}
	
	public void deleteFigure() {
		myFiguresList.remove(myUsedFigure);
		chooseFigure(-1);
		fillShapeList();
		repaint();
		myState = beforeFigureChosen;
		myChangaListener.frameChange();
	}
	
	private class MouseActionListener implements MouseListener  {
		public void mouseClicked(MouseEvent e) {
			int chosenFigure = searchChosenFigure(e.getPoint());
			if (chosenFigure == -1) {
				chooseFigure(-1);
				myState = beforeFigureChosen;
			} else {
				if (chosenFigure != myUsedFigure){
					chooseFigure(chosenFigure);
					setOldValueAndFillList(e.getPoint());
					myState = figureChosen;
					myChangaListener.frameChange();
				} else {
					myState = figureChosen;
				}
			}
		}
		
		public void mouseEntered(MouseEvent e) {
		}

		public void mouseExited(MouseEvent e) {
		}

		public void mousePressed(MouseEvent e) {
		}

		public void mouseReleased(MouseEvent e) {
			searchChosenFigure(e.getPoint());
			if (myUsedFigure == -1) {
				myState = new BeforeFigureChosen();
				repaint();
			} else {
				myState = new FigureChosen();
				repaint();
			}
		}		
	}
	
	
	private class MouseDraggedListener extends MouseMotionAdapter {
		public void mouseDragged(MouseEvent e) {	
			myState = myState.mouseDragged(e.getPoint());
		}
		
		public void mouseMoved(MouseEvent e) {
		}
	}
	
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;
		if (myUsedFigure != -1) {
			g2.setColor(Color.BLACK);
			if (myMovePointsShapeList == null) {
			}
			for (Shape movePointShape: myMovePointsShapeList) {
				g2.draw(movePointShape);
			}
		}
	}
		
	private interface IState {
		public IState mouseDragged(Point point);
	}
	

	private class BeforeFigureChosen implements IState {
		public IState mouseDragged(Point point) {
			myUsedFigure = searchChosenFigure(point);
			if (myUsedFigure != -1) {
				setOldValueAndFillList(point);
				return figureMove;
			}
			return beforeFigureChosen;
		}	
	}
	
	private class FigureChosen implements IState {
		public IState mouseDragged(Point point) {
			myPoint = searchChosenMovePoint(point);
			if (myPoint != -1) {
				myOldX = (int)point.getX();
				myOldY = (int)point.getY();
				myChangaListener.frameChange();
				return figureResize;
			}			
			int usedFigure = searchChosenFigure(point);	
			if (usedFigure != -1) {
				//choose another one
				if (myUsedFigure != usedFigure) {
					chooseFigure(usedFigure);
					setOldValueAndFillList(point);
					return figureMove;
				} 
				myOldX = (int)point.getX();
				myOldY = (int)point.getY();
				myChangaListener.frameChange();
				return figureMove;
			}
			chooseFigure(-1);
			return beforeFigureChosen;
		}	
	}
	
	private class FigureMove implements IState {
		public IState mouseDragged(Point point) {
			IGraphicalObject figure = myFiguresList.get(myUsedFigure);
			int dx = (int)point.getX() - myOldX;
			int dy = (int)point.getY() - myOldY;
			if (((figure instanceof RectangularFigure) &&
					!isInFieldRectangular(dx, dy, dx, dy)) ||
				(figure instanceof PolyFigure) && 
				!isInFieldPolyline(dx, dy, -1)) {
				fillMovePointsList();
				myOldX = (int)point.getX();
				myOldY = (int)point.getY();
				repaint();
				return figureMove;
			}
			figure.move((int)point.getX() - myOldX, (int)point.getY() - myOldY, 0, 0);
			for (Moviable movePoint: myMovePoints) {
				movePoint.move((int)point.getX() - myOldX, (int)point.getY() - myOldY, 0, 0);
			}
			fillMovePointsList();
			changeShapeInList(myUsedFigure);
			myOldX = (int)point.getX();
			myOldY = (int)point.getY();
			repaint();
			myChangaListener.frameChange();
			return figureMove;
		}
		
	}
	
	private class FigureResize implements IState {
		public IState mouseDragged(Point point) {
			IGraphicalObject figure = myFiguresList.get(myUsedFigure);
			int dx = (int)point.getX() - myOldX;
			int dy = (int)point.getY() - myOldY;
			if (figure instanceof PolyFigure) {
				if (isInFieldPolyline(dx, dy, myPoint)) {
					((PolyFigure)figure).movePoint(dx, dy, myPoint);
					myMovePoints.get(myPoint).move(dx, dy, 0, 0);					
				}
			} else {
				switch(myPoint) {
				case 0:
					if (!isResiziable(dx, dy, -1 * dx, -1 * dy) ||
							!isInFieldRectangular(dx, dy, -1 * dx, -1 * dy)) {
						break;
					}
					figure.move(dx, dy, - 1 * dx, -1 * dy);
					myMovePoints.get(0).move(dx, dy, 0, 0);
					myMovePoints.get(1).move(0, dy, 0, 0);
					myMovePoints.get(3).move(dx, 0, 0, 0);
					break;
				case 1:
					if (!isResiziable(0, dy, dx, -1 * dy) 
							|| !isInFieldRectangular(0, dy, dx, -1 * dy)) {
						break;
					}
					figure.move(0, dy, dx, -1 * dy);
					myMovePoints.get(0).move(0, dy, 0, 0);
					myMovePoints.get(1).move(dx, dy, 0, 0);
					myMovePoints.get(2).move(dx, 0, 0, 0);
					break;
				case 2:
					if (!isResiziable(0, 0, dx, dy) ||
							!isInFieldRectangular(0, 0, dx, dy)) {
						break;
					}
					figure.move(0, 0, dx, dy);
					myMovePoints.get(1).move(dx, 0, 0, 0);
					myMovePoints.get(2).move(dx, dy, 0, 0);
					myMovePoints.get(3).move(0, dy, 0, 0);
					break;
				case 3:
					if (!isResiziable(dx, 0, -1 * dx, dy) ||
							!isInFieldRectangular(dx, 0, -1 * dx, dy)) {
						break;
					}
					figure.move(dx, 0, -1 * dx, dy);
					myMovePoints.get(0).move(dx, 0, 0, 0);
					myMovePoints.get(2).move(0, dy, 0, 0);
					myMovePoints.get(3).move(dx, dy, 0, 0);
					break;
				}
			}
			fillMovePointsList();
			changeShapeInList(myUsedFigure);
			repaint();
			myOldX = (int)point.getX();
			myOldY = (int)point.getY();
			myChangaListener.frameChange();
			return figureResize;
		}
		
		private boolean isResiziable(int dx1, int dy1, int dx2, int dy2) {
			if ((myMovePoints.get(1).getPosition().getX() - 
					myMovePoints.get(0).getPosition().getX() + dx2 - dx1) < 4) {
				return false;
			}
			if ((myMovePoints.get(2).getPosition().getY() - 
					myMovePoints.get(1).getPosition().getY() + dy2 - dy1) < 4) {
				return false;
			}
			if ((myMovePoints.get(2).getPosition().getX() - 
					myMovePoints.get(3).getPosition().getX() + dx2 - dx1) < 4) {
				return false;
			}
			if ((myMovePoints.get(3).getPosition().getY() - 
					myMovePoints.get(0).getPosition().getY() + dy2 - dy1) < 4) {
				return false;
			}
			return true;
		}
		
	}
	
	 private boolean isInFieldRectangular(int dx1, int dy1, int dx2, int dy2) {
  	   int upperY = myMovePoints.get(0).getPosition().getY() + dy1;
  	   int bottomY = myMovePoints.get(3).getPosition().getY() + dy2;
  	   int leftX = myMovePoints.get(0).getPosition().getX() + dx1;
  	   int rightX = myMovePoints.get(1).getPosition().getX() + dx2;
  	   if ((upperY < 0) || (bottomY > myHeight)
  			   || (leftX < 0) || (rightX > myWidth)) {
  		   return false;
  		   }
		return true;						
	}
	 
	 private boolean isInFieldPolyline(int dx, int dy, int point) {
		 if (point > -1) {
			 int x = myMovePoints.get(point).getPosition().getX() + dx;
			 int y = myMovePoints.get(point).getPosition().getY() + dy;
			 if ((x > myWidth) || (x < 0) || (y > myHeight) || (y < 0)) {
				 return false;
			 }
		 } else {
			 int upper = myHeight;
			 int bottom = 0;
			 int left = myWidth;
			 int right = 0;
			 for (Moviable m :myMovePoints) {
				 int x = m.getPosition().getX() + dx;
				 if (x < left) {
					 left = x;					 
				 } else if (x > right) {
					 right = x;
				 }
				 
				 int y = m.getPosition().getY() + dy;
				 if (y > bottom) {
					 bottom = y;
				 } else if (y < upper) {
					 upper = y;					 
				 }
			 }
			 if ((upper  < 0) || (bottom > myHeight) || (left < 0) || (right > myWidth)) {
				 return false;
			 }
		 }
		 return true;
	 }
	
}
