package ru.amse.kanzheleva.moviemaker.ui.visitor;

import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

import ru.amse.kanzheleva.moviemaker.movie.IVisitor;
import ru.amse.kanzheleva.moviemaker.movie.Moviable;
import ru.amse.kanzheleva.moviemaker.movie.figures.IEllipse;
import ru.amse.kanzheleva.moviemaker.movie.figures.IPolyLine;
import ru.amse.kanzheleva.moviemaker.movie.figures.IPolygon;
import ru.amse.kanzheleva.moviemaker.movie.figures.IRectangle;
import ru.amse.kanzheleva.moviemaker.movie.figures.IRoundRectangle;

public class ShapeMaker implements IVisitor<Shape, Void> {
	private static ShapeMaker myInstance;
	
	private ShapeMaker() {
	}
	
	public static ShapeMaker getInstance() {
		if (myInstance == null) {
			myInstance = new ShapeMaker();
		}
		return myInstance;
	}
	public  Shape visit(IRectangle rectangle, Void s) {
		return new Rectangle2D.Double(rectangle.getPosition().getX(), 
				rectangle.getPosition().getY(), 
				rectangle.getWidth(), 
				rectangle.getHeight());
	}
	
	public Shape visit(IRoundRectangle rectangle, Void s) {
		return new RoundRectangle2D.Double(rectangle.getPosition().getX(), 
				rectangle.getPosition().getY(), 
				rectangle.getWidth(), 
				rectangle.getHeight(), 
				rectangle.getCurveWidth(),
				rectangle.getCurveHeight());
	}
	
	public Shape visit(IEllipse ellipse, Void s) {
		return new Ellipse2D.Double(ellipse.getPosition().getX(), 
				ellipse.getPosition().getY(),
				ellipse.getWidth(), 
				ellipse.getHeight());
	}
	
	public Shape visit(IPolygon polygon, Void s) {
		int[] x = polygon.getX();
		int[] y = polygon.getY();
		
		GeneralPath path = new GeneralPath(GeneralPath.WIND_EVEN_ODD, x.length);
		path.moveTo(x[0], y[0]);
		for (int i = 0; i < x.length; i++) {
			path.lineTo(x[i], y[i]);
		}
		path.closePath();
		
		return path;
	}
	
	public Shape visit(Moviable element, Void s) {
		return new Ellipse2D.Double(element.getPosition().getX() - 5, 
				element.getPosition().getY() - 5,
				10,10);
	}

	public Shape visit(IPolyLine polyLine, Void s) {
		int[] x = polyLine.getX();
		int[] y = polyLine.getY();
		
		GeneralPath path = new GeneralPath(GeneralPath.WIND_EVEN_ODD, x.length);
		path.moveTo(x[0], y[0] + 2);
		for (int i = 0; i < x.length; i++) {
			path.lineTo(x[i], y[i] + 2);
		}
		for (int i = x.length - 1; i >= 0; i--) {
			path.lineTo(x[i], y[i] - 2);
		}
		path.closePath();
		
		return path;
	}

}
