package ru.amse.kanzheleva.moviemaker.xml;

import org.xml.sax.Attributes;

import ru.amse.kanzheleva.moviemaker.movie.IMyPoint;
import ru.amse.kanzheleva.moviemaker.movie.IVisitor;
import ru.amse.kanzheleva.moviemaker.movie.Moviable;
import ru.amse.kanzheleva.moviemaker.movie.MyPoint;
import ru.amse.kanzheleva.moviemaker.movie.figures.IEllipse;
import ru.amse.kanzheleva.moviemaker.movie.figures.IPolyFigure;
import ru.amse.kanzheleva.moviemaker.movie.figures.IPolyLine;
import ru.amse.kanzheleva.moviemaker.movie.figures.IPolygon;
import ru.amse.kanzheleva.moviemaker.movie.figures.IRectangle;
import ru.amse.kanzheleva.moviemaker.movie.figures.IRectangularFigure;
import ru.amse.kanzheleva.moviemaker.movie.figures.IRoundRectangle;

public class FigureReader implements IVisitor<Void, Attributes> {
	
	public Void visit(IEllipse ellipse, Attributes attributes) {
		readRectangularReader(ellipse, attributes);
		return null;
	}
	
	public Void visit(IRectangle rectangle,Attributes attributes) {
		readRectangularReader(rectangle, attributes);
		return null;
	}
	
	public Void visit(IRoundRectangle roundRectangle, Attributes attributes) {
		readRectangularReader(roundRectangle, attributes);
		roundRectangle.setCurveWidth(
				Integer.valueOf(attributes.getValue(MovieConstants.CURVEWIDTH)));
		roundRectangle.setCurveHeight(
				Integer.valueOf(attributes.getValue(MovieConstants.CURVEHEIGHT)));
		return null;
	}
	
	public Void visit(IPolygon polygon, Attributes attributes) {
		readPolygoneAndPolyLine(polygon, attributes);
		return null;
	}
	
	private void readRectangularReader(IRectangularFigure figure, Attributes attributes) {
		figure.setWidth(readWidth(attributes));
		figure.setHeight(readHeight(attributes));
		figure.setPosition(readPoint(attributes));
	}
	
	private int readWidth(Attributes attributes) {
		return Integer.valueOf(attributes.getValue(MovieConstants.WIDTH));
	}
	
	private int readHeight(Attributes attributes) {
		return Integer.valueOf(attributes.getValue(MovieConstants.HEIGHT));
	}
	
	private IMyPoint readPoint(Attributes attributes) {
		IMyPoint point = new MyPoint(
				Integer.valueOf(attributes.getValue(MovieConstants.X)),
				Integer.valueOf(attributes.getValue(MovieConstants.Y)));
		return point;
	}

	public Void visit(Moviable element, Attributes attributes) {
		return null;
	}

	public Void visit(IPolyLine polyLine, Attributes attributes) {
		readPolygoneAndPolyLine(polyLine, attributes);
		return null;
	}

	private void readPolygoneAndPolyLine(IPolyFigure polyFigure, Attributes attributes) {
		int size = Integer.valueOf(attributes.getValue(MovieConstants.N));
		int[] x = new int[size];
		int[] y = new int[size];
		for (int i = 0; i < size; i++) {
			x[i] = Integer.valueOf(attributes.getValue(MovieConstants.X + i));
			y[i] = Integer.valueOf(attributes.getValue(MovieConstants.Y + i));
		}
		polyFigure.setX(x);
		polyFigure.setY(y);
	}
}
