package ru.amse.kanzheleva.moviemaker.xml;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import ru.amse.kanzheleva.moviemaker.movie.IGraphicalObject;
import ru.amse.kanzheleva.moviemaker.movie.IVisitor;
import ru.amse.kanzheleva.moviemaker.movie.Moviable;
import ru.amse.kanzheleva.moviemaker.movie.figures.IEllipse;
import ru.amse.kanzheleva.moviemaker.movie.figures.IPolyFigure;
import ru.amse.kanzheleva.moviemaker.movie.figures.IPolyLine;
import ru.amse.kanzheleva.moviemaker.movie.figures.IPolygon;
import ru.amse.kanzheleva.moviemaker.movie.figures.IRectangle;
import ru.amse.kanzheleva.moviemaker.movie.figures.IRectangularFigure;
import ru.amse.kanzheleva.moviemaker.movie.figures.IRoundRectangle;

public class FigureSaver implements IVisitor<Void, XMLStreamWriter> {
	public Void visit(IRectangle rectangle, XMLStreamWriter writer) {
		try {
			writer.writeStartElement(MovieConstants.RECTANGLE);
			writeIGraphicalObjectAttributes(writer, rectangle);
			writeRectangularAttributes(writer, rectangle);
			writer.writeEndElement();
		} catch (XMLStreamException e) {
			throw new WrongDataException("wrong Data");
		}
		return null;
	}

	public Void visit(IRoundRectangle rectangle, XMLStreamWriter writer) {
		try {
			writer.writeStartElement(MovieConstants.ROUNDRECTANGLE);
			writeIGraphicalObjectAttributes(writer, rectangle);
			writeRectangularAttributes(writer, rectangle);
			writer.writeAttribute(MovieConstants.CURVEWIDTH, rectangle.getCurveWidth() + "");
			writer.writeAttribute(MovieConstants.CURVEHEIGHT, rectangle.getCurveHeight() + "");
			writer.writeEndElement();
		} catch (XMLStreamException e) {
			throw new WrongDataException("wrong Data");
		}
		return null;
	}

	public Void visit(IEllipse ellipse, XMLStreamWriter writer) {
		try {
			writer.writeStartElement(MovieConstants.ELLIPSE);
			writeIGraphicalObjectAttributes(writer, ellipse);
			writeRectangularAttributes(writer, ellipse);
			writer.writeEndElement();
		} catch (XMLStreamException e) {
			throw new WrongDataException("wrong Data");
		}
		return null;
	}

	public Void visit(IPolygon polygon, XMLStreamWriter writer) {
		try {
			writer.writeStartElement(MovieConstants.POLYGON);
			writeIGraphicalObjectAttributes(writer, polygon);
			writePolyFigureAttributes(writer, polygon);
			writer.writeEndElement();
		} catch (XMLStreamException e) {
			throw new WrongDataException("wrong Data");
		}
		return null;
	}

	public Void visit(IPolyLine polyLine, XMLStreamWriter writer) {
		try {
			writer.writeStartElement(MovieConstants.POLYLINE);
			writeIGraphicalObjectAttributes(writer, polyLine);
			writePolyFigureAttributes(writer, polyLine);
			writer.writeEndElement();
		} catch (XMLStreamException e) {
			throw new WrongDataException("wrong Data");
		}
		return null;
	}

	public Void visit(Moviable element, XMLStreamWriter writer) {
		return null;
	}
	
	
	private void writeRectangularAttributes(XMLStreamWriter writer, IRectangularFigure element) throws XMLStreamException {
		writer.writeAttribute(MovieConstants.WIDTH, element.getWidth() + "");
		writer.writeAttribute(MovieConstants.HEIGHT, element.getHeight() + "");
		writer.writeAttribute(MovieConstants.X, element.getPosition().getX() + "");
		writer.writeAttribute(MovieConstants.Y, element.getPosition().getY() + "");		
	}
	
	private void writeIGraphicalObjectAttributes(XMLStreamWriter writer, IGraphicalObject element) throws XMLStreamException {
		writer.writeAttribute(MovieConstants.FILLRED, element.getFillColor().getRed() + "");
		writer.writeAttribute(MovieConstants.FILLGREEN, element.getFillColor().getGreen() + "");
		writer.writeAttribute(MovieConstants.FILLBLUE, element.getFillColor().getBlue() + "");
		
		writer.writeAttribute(MovieConstants.LINERED, element.getLineColor().getRed() + "");
		writer.writeAttribute(MovieConstants.LINEGREEN, element.getLineColor().getGreen() + "");
		writer.writeAttribute(MovieConstants.LINEBLUE, element.getLineColor().getBlue() + "");
		
		writer.writeAttribute(MovieConstants.ROTATIONANGLE, element.getRotationAngle() + "");
		writer.writeAttribute(MovieConstants.XCENTER, element.getRotationCenter().getX() + "");
		writer.writeAttribute(MovieConstants.YCENTER, element.getRotationCenter().getY() + "");
	}

	private void writePolyFigureAttributes(XMLStreamWriter writer, IPolyFigure element) throws XMLStreamException {
		int[] x = element.getX();
		int[] y = element.getY();
		int n = x.length;
		writer.writeStartElement(MovieConstants.POINTS);
		writer.writeAttribute(MovieConstants.N, n + "");
		for (int i = 0; i < n; i++) {
			writer.writeAttribute(MovieConstants.X + i, x[i] + "");
			writer.writeAttribute(MovieConstants.Y + i, y[i] + "");
		}
		writer.writeEndElement();
		
	}
}
