package ru.amse.kanzheleva.moviemaker.xml;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import ru.amse.kanzheleva.moviemaker.movie.IFrame;
import ru.amse.kanzheleva.moviemaker.movie.IGraphicalObject;
import ru.amse.kanzheleva.moviemaker.movie.IMovie;



public class MovieSaver {
	public void write(String fileName, IMovie movie) throws WrongDataException {
		OutputStream stream = null;
		try {
			stream = new FileOutputStream(fileName);
			XMLOutputFactory factory = XMLOutputFactory.newInstance();
			XMLStreamWriter writer = factory.createXMLStreamWriter(stream);
			
			writer.writeStartDocument();
			List<IFrame> frameList = movie.getFrameList();
			writer.writeStartElement(MovieConstants.MOVIE);
			writer.writeAttribute(MovieConstants.FPS, movie.getFPS() + "");
			for (IFrame frame: frameList) {
				writer.writeStartElement(MovieConstants.FRAME);
				List<IGraphicalObject> graphicalObjectList =  frame.getGraphicalObjects();
				for (IGraphicalObject element : graphicalObjectList) {
					element.visit(new FigureSaver(), writer);
				}			
				writer.writeEndElement();
			}
			writer.writeEndElement();
			writer.writeEndDocument();
			writer.close();
		} catch (XMLStreamException e) {
			throw new WrongDataException("XMLIO problem");
		} catch (FileNotFoundException e) {
			throw new WrongDataException("IO problem");
		} finally {
			try {
				if (stream != null) {
					stream.close();	
				}
			} catch (IOException e) {
				throw new WrongDataException("IO problem");
			}		
		}
	}
}
